@@ -23,15 +23,16 @@ pub fn braille_mathml(mathml: Element, nav_node_id: String) -> Result<String> {
             // FIX: need to set name of speech rules so test Nemeth/UEB clean for
         let pref_manager = rules_with_context.get_rules().pref_manager.borrow();
         let highlight_style = pref_manager.get_user_prefs().to_string("BrailleNavHighlight");
-        let braille = if &pref_manager.get_user_prefs().to_string("BrailleCode") == "UEB" {
-            ueb_cleanup(braille_string.replace(" ", ""))
+        let braille_code = &pref_manager.get_user_prefs().to_string("BrailleCode");
+        let braille = if braille_code == "UEB" {
+            ueb_cleanup(braille_string.replace(' ', ""))
         } else {
-            nemeth_cleanup(braille_string.replace(" ", ""))
+            nemeth_cleanup(braille_string.replace(' ', ""))
         };
 
         return Ok(
             if highlight_style != "Off" {
-                highlight_braille_chars(braille, highlight_style == "All")
+                highlight_braille_chars(braille, braille_code, highlight_style == "All")
             } else {
              braille
             }
@@ -41,25 +42,26 @@ pub fn braille_mathml(mathml: Element, nav_node_id: String) -> Result<String> {
     // highlight with dots 7 & 8 based on the highlight style
     // both the start and stop points will be extended to deal with indicators such as capitalization
     // if 'fill_range' is true, the interior will be highlighted
-    fn highlight_braille_chars(braille: String, fill_range: bool) -> String {
+    fn highlight_braille_chars(braille: String, braille_code: &str, fill_range: bool) -> String {
         let mut braille = braille;
-        // some special chars weren't converted to having dots 7 & 8 to indicate navigation position -- add them
+        // some special (non-braille) chars weren't converted to having dots 7 & 8 to indicate navigation position
+        // they need to be added to the start
 
-        // find start and end indexes
-        let start = braille.find(|ch| is_highlighted(ch));
-        let end = braille.rfind(|ch| is_highlighted(ch));
+        // find start and end indexes of the highlighted region
+        let start = braille.find(is_highlighted);
+        let end = braille.rfind(is_highlighted);
         if start.is_none() {
             assert!(end.is_none());
             return braille;
         };
 
-        let start = start.unwrap();
-        let end = end.unwrap();
+        let end = end.unwrap();         // always exists if start exists
+        let start = highlight_first_indicator(&mut braille, braille_code, start.unwrap(), end);
 
-        highlight_first_indicator(&mut braille, start);
         if start == end {
             return braille;
         }
+
         if !fill_range {
             return braille;
         }
@@ -73,67 +75,131 @@ pub fn braille_mathml(mathml: Element, nav_node_id: String) -> Result<String> {
         result.push_str(&braille[end..]);
         return result;
 
-        fn is_highlighted(ch: char) -> bool {
-            let ch_as_u32 = ch as u32;
-            return 0x28C0 <= ch_as_u32 && ch_as_u32 <= 0x28FF;
-        }
+        fn highlight_first_indicator(braille: &mut String, braille_code: &str, start_index: usize, end_index: usize) -> usize {
+            // chars in the braille block range use 3 bytes -- we can use that to optimize the code some
+            let first_ch = unhighlight(braille[start_index..start_index+3].chars().next().unwrap());
 
-        fn highlight(ch: char) -> char {
-            return unsafe{char::from_u32_unchecked(ch as u32 | 0xC0)};      
-        }
-
-        fn unhighlight(ch: char) -> char {
-            return unsafe{char::from_u32_unchecked(ch as u32 & 0x283F)};      
-        }
-
-        fn highlight_first_indicator(braille: &mut String, ch_index: usize) {
-            // need to highlight (optional) capital/number, language, and style also in that (rev) order
-            // chars in the braille block range use 3 bytes
-            let mut n_bytes = ch_index;     // how far to move back
-            let prefix_ch_index = std::cmp::max(0, ch_index as isize - 12) as usize;
-            let indicators = &braille[prefix_ch_index..ch_index];   // chars to be examined
-            let prefix = &mut indicators.chars().rev().peekable();
-            if prefix.peek() == Some(&&'⠠') { // cap indicator
-                n_bytes -= 3;
-                prefix.next();
-            } else if prefix.peek() == Some(&&'⠼') { // number indicator
-                n_bytes -= 3;
-                prefix.next();
-            } 
-            if [Some(&'⠸'), Some(&'⠈'), Some(&'⠨')].contains(&prefix.peek()) { // bold, script/blackboard, italic indicator
-                n_bytes -= 3;
-                prefix.next();
-            }
-
-            if [Some(&'⠰'), Some(&'⠸'), Some(&'⠨')].contains(&prefix.peek()) {   // English, German, Greek
-                n_bytes -= 3;
-            } else if prefix.peek() == Some(&&'⠈') {  
-                let ch = prefix.next();                              // Russian/Greek Variant
-                if ch == Some('⠈') || ch == Some('⠨') {
-                    n_bytes -= 6;
-                }
-            } else if prefix.peek() == Some(&&'⠠')  { // Hebrew 
-                let ch = prefix.next();                              // Russian/Greek Variant
-                if ch == Some('⠠') {
-                    n_bytes -= 6;
-                }
+            // need to highlight (optional) capital/number, language, and style (max 2 chars) also in that (rev) order
+            let prefix_ch_index = std::cmp::max(0, start_index as isize - 5*3) as usize;
+            let indicators = &braille[prefix_ch_index..start_index];   // chars to be examined
+            let i_byte_start = start_index - 3 * match braille_code {
+                "Nemeth" => i_start_nemeth(indicators, first_ch),
+                "UEB" => i_start_ueb(indicators),
+                _ => {
+                    error!("highlight_first_indicator: Unknown braille code '{}'", braille);
+                    0
+                },
             };
-            if n_bytes < ch_index {
-                // remove old highlight
-                let replacement_range = ch_index..ch_index+3;
-                let replacement_str = unhighlight(braille[replacement_range.clone()].chars().next().unwrap()).to_string();
-                braille.replace_range(replacement_range, &replacement_str);
+            if i_byte_start < start_index {
+                // remove old highlight as long as we don't wipe out the end highlight
+                if start_index < end_index {
+                    let old_first_char_bytes = start_index..start_index+3;
+                    let replacement_str = unhighlight(braille[old_first_char_bytes.clone()].chars().next().unwrap()).to_string();
+                    braille.replace_range(old_first_char_bytes, &replacement_str);
+                }
 
                 // add new highlight
-                let replacement_range = n_bytes..n_bytes+3;
-                let replacement_str = highlight(braille[replacement_range.clone()].chars().next().unwrap()).to_string();
-                braille.replace_range(replacement_range, &replacement_str);
+                let new_first_char_bytes = i_byte_start..i_byte_start+3;
+                let replacement_str = highlight(braille[new_first_char_bytes.clone()].chars().next().unwrap()).to_string();
+                braille.replace_range(new_first_char_bytes, &replacement_str);
             }
+
+            return i_byte_start;
         }
 
     }
+
+
+    fn i_start_nemeth(braille_prefix: &str, first_ch: char) -> usize {
+        static NEMETH_NUMBERS: phf::Set<char> = phf_set! {
+            '⠂', '⠆', '⠒', '⠲', '⠢', '⠖', '⠶', '⠦', '⠔', '⠴', '⠨' // 1, 2, ...9, 0, decimal pt
+        };
+        let mut n_chars = 0;
+        let prefix = &mut braille_prefix.chars().rev().peekable();
+        if prefix.peek() == Some(&'⠠') ||  // cap indicator
+           (prefix.peek() == Some(&'⠼') && NEMETH_NUMBERS.contains(&first_ch)) ||  // number indicator
+           [Some(&'⠸'), Some(&'⠈'), Some(&'⠨')].contains(&prefix.peek()) {         // bold, script/blackboard, italic indicator
+            n_chars += 1;
+            prefix.next();
+        } 
+
+        if [Some(&'⠰'), Some(&'⠸'), Some(&'⠨')].contains(&prefix.peek()) {   // English, German, Greek
+            n_chars += 1;
+        } else if prefix.peek() == Some(&'⠈') {  
+            let ch = prefix.next();                              // Russian/Greek Variant
+            if ch == Some('⠈') || ch == Some('⠨') {
+                n_chars += 2;
+            }
+        } else if prefix.peek() == Some(&'⠠')  { // Hebrew 
+            let ch = prefix.next();                              // Russian/Greek Variant
+            if ch == Some('⠠') {
+                n_chars += 2;
+            }
+        };
+        return n_chars;
+    }
+
+    fn i_start_ueb(braille_prefix: &str) -> usize {
+        static UEB_PREFIXES: phf::Set<char> = phf_set! {
+            '⠼', '⠈', '⠘', '⠸', '⠐', '⠨', '⠰', '⠠',
+        };
+
+        let prefix = &mut braille_prefix.chars().rev().peekable();
+        let mut n_chars = 0;
+        while let Some(ch) = prefix.next() {
+            if UEB_PREFIXES.contains(&ch) {
+                n_chars += 1;
+            } else if ch == '⠆' {
+                let n_typeform_chars = check_for_typeform(prefix);
+                if n_typeform_chars > 0 {
+                    n_chars += n_typeform_chars;
+                } else {
+                    break;
+                }
+            } else {
+                break;
+            }
+        }
+        return n_chars;
+    }
+
+    fn check_for_typeform(prefix: &mut dyn std::iter::Iterator<Item=char>) -> usize {
+        static UEB_TYPEFORM_PREFIXES: phf::Set<char> = phf_set! {
+            '⠈', '⠘', '⠸', '⠨',
+        };
+
+        if let Some(typeform_indicator) = prefix.next() {
+            if UEB_TYPEFORM_PREFIXES.contains(&typeform_indicator) {
+                return 2;
+            } else if typeform_indicator == '⠼' {
+                if let Some(user_defined_typeform_indicator) = prefix.next() {
+                    if UEB_TYPEFORM_PREFIXES.contains(&user_defined_typeform_indicator) || user_defined_typeform_indicator == '⠐' {
+                        return 3;
+                    }
+                }
+            }
+        }
+        return 0;
+    }
 }
 
+fn is_highlighted(ch: char) -> bool {
+    let ch_as_u32 = ch as u32;
+    return (0x28C0..0x28FF).contains(&ch_as_u32);
+}
+
+fn highlight(ch: char) -> char {
+    return unsafe{char::from_u32_unchecked(ch as u32 | 0xC0)};      
+}
+
+fn unhighlight(ch: char) -> char {
+    let ch_as_u32 = ch as u32;
+    if (0x28C0..0x28FF).contains(&ch_as_u32) {
+        return unsafe{char::from_u32_unchecked(ch_as_u32 & 0x283F)};  
+    } else {
+        return ch;
+    }
+}
 
 
 fn nemeth_cleanup(raw_braille: String) -> String {
@@ -284,7 +350,7 @@ fn nemeth_cleanup(raw_braille: String) -> String {
 
     // Remove unicode blanks at start and end -- do this after the substitutions because ',' introduces spaces
     let result = result.trim_start_matches('⠀').trim_end_matches('⠀');
-    let result = COLLAPSE_SPACES.replace_all(&result, "⠀");
+    let result = COLLAPSE_SPACES.replace_all(result, "⠀");
    
     return result.to_string();
 
@@ -385,7 +451,7 @@ fn ueb_cleanup(raw_braille: String) -> String {
 
     
     // '𝐖' is a hard break -- basically, it separates exprs
-    let mut result = result.split("𝐖")
+    let mut result = result.split('𝐖')
                         .map(|str| pick_start_mode(str) + "W")
                         .collect::<String>();
     result.pop();   // we added a 'W' at the end that needs to be removed.
@@ -511,7 +577,7 @@ fn ueb_cleanup(raw_braille: String) -> String {
                 i += 3 // eat "C", etc
             } else if ch == 'L' {       // must be lowercase -- uppercase consumed above
                 if !word_mode_end.is_empty() {
-                    assert!(LETTERS.contains(&chars[i+1]));
+                    assert!(LETTERS.contains(&unhighlight(chars[i+1])));
                     // add terminator if terminated by lowercase letter
                     for ch in word_mode_end.chars() {
                         result.push(ch);
@@ -534,18 +600,17 @@ fn ueb_cleanup(raw_braille: String) -> String {
 
     fn is_next_char(chars: &[char], target: char) -> bool {
         // first find the L or N and eat the char so that we are at the potential start of where the target lies
-            debug!("Looking for '{}' in '{}'", target, chars.iter().collect::<String>());
+            // debug!("Looking for '{}' in '{}'", target, chars.iter().collect::<String>());
         for i_end in 0..chars.len() {
             if chars[i_end] == 'L' || chars[i_end] == 'N' {
                 // skip the next char to get to the real start, and then look for the target
                 // stop when L/N signals past potential target or we hit some non L/N char (actual braille)
-                debug!("   after L/N '{}'", chars[i_end+2..].iter().collect::<String>());
-                for i_next in i_end+2..chars.len() {
-                    let ch = chars[i_next];
+                // debug!("   after L/N '{}'", chars[i_end+2..].iter().collect::<String>());
+                for &ch in chars.iter().skip(i_end+2) {
                     if ch == 'L' || ch == 'N' || !LETTER_PREFIXES.contains(&ch) {
                         return false;
                     } else if ch == target {
-                        debug!("   found target");
+                        // debug!("   found target");
                         return true;
                     }
                 }
@@ -563,25 +628,25 @@ fn ueb_cleanup(raw_braille: String) -> String {
         //   begin the expression with a grade 1 word indicator (or a passage indicator if the expression includes spaces)
         // Apparently "only a grade 1 symbol..." means at most one grade 1 symbol based on some examples (GTM 6.4, example 4)
         debug!("before determining mode:  '{}'", raw_braille);
-        let grade2 = remove_unneeded_mode_changes(&raw_braille, UEB_Mode::Grade2, UEB_Duration::Symbol);
-        debug!("Symbol mode:  '{}'", &grade2);
+        let grade2 = remove_unneeded_mode_changes(raw_braille, UEB_Mode::Grade2, UEB_Duration::Symbol);
+        debug!("Symbol mode:  '{}'", grade2);
 
         if is_grade2_string_ok(&grade2) {
             return grade2;
         } else {
-            let grade1_word = remove_unneeded_mode_changes(&raw_braille, UEB_Mode::Grade1, UEB_Duration::Word);
-            debug!("Word mode:    '{}'", &grade1_word);
+            let grade1_word = remove_unneeded_mode_changes(raw_braille, UEB_Mode::Grade1, UEB_Duration::Word);
+            debug!("Word mode:    '{}'", grade1_word);
             
             // BANA says use g1 word mode if spaces are present, but that's not what their examples do
-            // A conversation with a BANA said that they mean use passage mode if ≥3 "segments" (≥2 blanks)
+            // A conversation with Ms. DeAndrea from BANA said that they mean use passage mode if ≥3 "segments" (≥2 blanks)
             let mut n_blanks = 0;
-            if grade1_word.chars().find(|&ch| {
+            if grade1_word.chars().any(|ch| {
                 if ch == 'W' {
                     n_blanks += 1;
                 }
                 n_blanks == 2
-            }).is_some() {
-                let grade1_passage = remove_unneeded_mode_changes(&raw_braille, UEB_Mode::Grade1, UEB_Duration::Passage);
+            }) {
+                let grade1_passage = remove_unneeded_mode_changes(raw_braille, UEB_Mode::Grade1, UEB_Duration::Passage);
                 debug!("Passage mode: '{}'", &grade1_passage);
                 return "⠰⠰⠰".to_string() + &grade1_passage + "⠰⠄";
             } else {
@@ -636,12 +701,12 @@ fn ueb_cleanup(raw_braille: String) -> String {
             // A '1' is forced if 'a-j' follows a digit
             assert_eq!(chars[i], '1', "'is_forced_grade1' didn't start with '1'");
             // check that a-j follows the '1'
-            if i+1 < chars.len() && LETTER_NUMBERS.contains(&chars[i+1]) {
+            if i+1 < chars.len() && LETTER_NUMBERS.contains(&unhighlight(chars[i+1])) {
                 // check for a number before the '1'
                 // this will be 'N' followed by LETTER_NUMBERS or the number ".", ",", or " "
                 for j in (0..i).rev() {
                     let ch = chars[j];
-                    if !(LETTER_NUMBERS.contains(&ch) || ".,W𝐖".contains(ch)) {
+                    if !(LETTER_NUMBERS.contains(&unhighlight(ch)) || ".,W𝐖".contains(ch)) {
                         return ch == 'N'
                     }
                 }
@@ -733,12 +798,12 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                 //   grade 1 indicators are not used unless a single lower-case letter a-j immediately follows a digit.
                 // Grade 1 mode when set by the numeric indicator is terminated by a space, hyphen, dash, or a grade 1 indicator.
                 i_g2_start = None;
-                debug!("Numeric: ch={}, duration: {:?}", ch, duration);
+                // debug!("Numeric: ch={}, duration: {:?}", ch, duration);
                 match ch {
                     'L' => {
                         // terminate numeric mode -- duration doesn't change
                         // let the default case handle pushing on the chars for the letter
-                        if LETTER_NUMBERS.contains(&chars[i+1]) {
+                        if LETTER_NUMBERS.contains(&unhighlight(chars[i+1])) {
                             result.push('1');   // need to distinguish a-j from a digit
                         }
                         result.push(ch);
@@ -757,7 +822,7 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                     '#' => {
                         // terminate numeric mode -- duration doesn't change
                         i += 1;
-                        if i+1 < chars.len() && chars[i] == 'L' && LETTER_NUMBERS.contains(&chars[i+1]) {
+                        if i+1 < chars.len() && chars[i] == 'L' && LETTER_NUMBERS.contains(&unhighlight(chars[i+1])) {
                             // special case where the script was numeric and a letter follows, so need to put out G1 indicator
                             result.push('1');
                             // the G1 case should work with 'L' now
@@ -786,13 +851,13 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                 // Grade 1 mode, when initiated by the numeric indicator, is terminated by a space, hyphen, dash or grade 1 terminator.
                 // Grade 1 mode is also set by grade 1 indicators.
                 i_g2_start = None;
-                debug!("Grade 1: ch={}, duration: {:?}", ch, duration);
+                // debug!("Grade 1: ch={}, duration: {:?}", ch, duration);
                 match ch {
                     'L' => {
                         // note: be aware of '#' case for Numeric because '1' might already be generated
                         // let prev_ch = if i > 1 {chars[i-1]} else {'1'};   // '1' -- anything beside ',' or '.'
                         // if duration == UEB_Duration::Symbol || 
-                        //     ( ",. ".contains(prev_ch) && LETTER_NUMBERS.contains(&chars[i+1]) ) {
+                        //     ( ",. ".contains(prev_ch) && LETTER_NUMBERS.contains(&unhighlight(chars[i+1])) ) {
                         //     result.push('1');        // need to retain grade 1 indicator (RUEB 6.5.2)
                         // }
                         // let the default case handle pushing on the chars for the letter
@@ -838,7 +903,7 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                    i_g2_start = Some(i);
                    cap_word_mode = false;
                }
-                debug!("Grade 2: ch={}, duration: {:?}", ch, duration);
+                // debug!("Grade 2: ch={}, duration: {:?}", ch, duration);
                 match ch {
                     'L' => {
                         if start_g2_letter.is_none() {
@@ -847,11 +912,11 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                         let (is_alone, right_matched_chars, n_letters) = stands_alone(&chars, i);
                         // GTM 1.2.1 says we only need to use G1 for single letters or sequences that are a shortform (e.g, "ab")
                         if is_alone && (n_letters == 1 || is_short_form(&right_matched_chars[..2*n_letters])) {
-                            debug!("  is_alone -- pushing '1'");
+                            // debug!("  is_alone -- pushing '1'");
                             result.push('1');
                             mode = UEB_Mode::Grade1;
                         }
-                        debug!("  pushing {:?}", right_matched_chars);
+                        // debug!("  pushing {:?}", right_matched_chars);
                         right_matched_chars.iter().for_each(|&ch| result.push(ch));
                         i += right_matched_chars.len();
                     },
@@ -866,7 +931,7 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                             let (is_alone, right_matched_chars, n_letters) = stands_alone(&chars, if is_greek {i+2} else {i+1});
                             // GTM 1.2.1 says we only need to use G1 for single letters or sequences that are a shortform (e.g, "ab")
                             if is_alone && (n_letters == 1 || is_short_form(&right_matched_chars[..2*n_letters])) {
-                                debug!("  is_alone -- pushing '1'");
+                                // debug!("  is_alone -- pushing '1'");
                                 result.push('1');
                                 mode = UEB_Mode::Grade1;
                             }
@@ -879,7 +944,7 @@ fn remove_unneeded_mode_changes(raw_braille: &str, start_mode: UEB_Mode, start_d
                                 i += 1;
                             }
                             start_g2_letter = Some(i);
-                            debug!("  pushing 'C' + {:?}", right_matched_chars);
+                            // debug!("  pushing 'C' + {:?}", right_matched_chars);
                             right_matched_chars.iter().for_each(|&ch| result.push(ch));
                             i += 1 + right_matched_chars.len();
                         }
@@ -967,10 +1032,9 @@ fn stands_alone(chars: &[char], i: usize) -> (bool, &[char], usize) {
             i -= 1;
             let ch = chars[i];
             let prev_ch = if i > 0 {chars[i-1]} else {' '};  // ' ' is a char not in input
-            debug!("  left alone: prev/ch {}/{}", prev_ch, ch);
-            if !intervening_chars_mode && prev_ch == 'L' {
-                i -= 1;       // ignore 'Lx' and also ignore 'ox'
-            } else if ch == 'o' || ch == 'b' {
+            // debug!("  left alone: prev/ch {}/{}", prev_ch, ch);
+            if (!intervening_chars_mode && prev_ch == 'L') ||
+               (ch == 'o' || ch == 'b') {
                 i -= 1;       // ignore 'Lx' and also ignore 'ox'
             } else if LEFT_INTERVENING_CHARS.contains(&ch) {
                 intervening_chars_mode = true;
@@ -997,7 +1061,7 @@ fn stands_alone(chars: &[char], i: usize) -> (bool, &[char], usize) {
         let mut n_letters = 1;      // we have skipped the first letter
         while i < chars.len() {
             let ch = chars[i];
-            debug!("  right alone: ch/next {}/{}", ch, if i+1<chars.len() {chars[i+1]} else {' '});
+            // debug!("  right alone: ch/next {}/{}", ch, if i+1<chars.len() {chars[i+1]} else {' '});
             if !intervening_chars_mode && ch == 'L' {
                 n_letters += 1;
                 i += 1;       // ignore 'Lx' and also ignore 'ox'
@@ -1054,15 +1118,15 @@ fn handle_contractions(chars: &[char], mut result: String) -> String {
     }
 
     let mut chars_as_str = chars.iter().collect::<String>();
-    debug!("  handle_contractions: examine '{}'", &chars_as_str);
+    // debug!("  handle_contractions: examine '{}'", &chars_as_str);
     let matches = CONTRACTION_PATTERNS.matches(&chars_as_str);
     for i in matches.iter() {
         let element = &CONTRACTIONS[i];
-        debug!("  replacing '{}' with '{}' in '{}'", element.pattern, element.replacement, &chars_as_str);
+        // debug!("  replacing '{}' with '{}' in '{}'", element.pattern, element.replacement, &chars_as_str);
         result.truncate(result.len() - chars_as_str.len());
         chars_as_str = CONTRACTION_REGEX[i].replace_all(&chars_as_str, element.replacement).to_string();
         result.push_str(&chars_as_str);
-        debug!("  result after replace '{}'", result);
+        // debug!("  result after replace '{}'", result);
     }
     return result;
 
@@ -1097,14 +1161,14 @@ use sxd_xpath::function::Error as XPathError;
 use std::result::Result as StdResult;
 
 pub struct NemethNestingChars;
-const NEMETH_FRAC_LEVEL: &'static str = "nemeth-frac-level";    // name of attr where value is cached
+const NEMETH_FRAC_LEVEL: &str = "nemeth-frac-level";    // name of attr where value is cached
 const FIRST_CHILD_ONLY: &[&str] = &["mroot", "msub", "msup", "msubsup", "munder", "mover", "munderover", "mmultiscripts"];
 impl NemethNestingChars {
     // returns a 'repeat_char' corresponding to the Nemeth rules for nesting
     // note: this value is likely one char too long because the starting fraction is counted
     fn nemeth_frac_value<'a>(node: &'a Element, repeat_char: &'a str) -> String {
         let children = node.children();
-        let name = name(&node);
+        let name = name(node);
         if is_leaf(*node) {
             return "".to_string();
         } else if name == "mfrac" {
@@ -1203,7 +1267,7 @@ impl BrailleChars {
     // returns a string for the chars in the *leaf* node.
     // this string follows the Nemeth rules typefaces and deals with mathvariant
     //  which has partially turned chars to the alphanumeric block
-    fn get_braille_chars<'a>(node: &'a Element, code: &str, text_range: Option<Range<usize>>) -> StdResult<String, XPathError> {
+    fn get_braille_chars(node: &Element, code: &str, text_range: Option<Range<usize>>) -> StdResult<String, XPathError> {
         match code {
             "Nemeth" => return BrailleChars::get_braille_nemeth_chars(node, text_range),
             "UEB" => return BrailleChars:: get_braille_ueb_chars(node, text_range),
@@ -1214,7 +1278,7 @@ impl BrailleChars {
         };
     }
 
-    fn get_braille_nemeth_chars<'a>(node: &'a Element, text_range: Option<Range<usize>>) -> StdResult<String, XPathError> {
+    fn get_braille_nemeth_chars(node: &Element, text_range: Option<Range<usize>>) -> StdResult<String, XPathError> {
         lazy_static! {
             // To greatly simplify typeface/language generation, the chars have unique ASCII chars for them:
             // Typeface: S: sans-serif, B: bold, 𝔹: blackboard, T: script, I: italic, R: Roman
@@ -1239,7 +1303,7 @@ impl BrailleChars {
             },
         };
         let text = BrailleChars::substring(as_text(*node), text_range);
-        let braille_chars = crate::speech::braille_replace_chars(&text, *node).unwrap_or("".to_string());
+        let braille_chars = crate::speech::braille_replace_chars(&text, *node).unwrap_or_else(|_| "".to_string());
         // debug!("braille_chars: '{}'", braille_chars);
         
         // we want to pull the prefix (typeface, language) out to the front until a change happens
@@ -1254,7 +1318,7 @@ impl BrailleChars {
             //        &caps["face"], &caps["lang"], &caps["num"], &caps["cap"], &caps["char"]);
             let mut nemeth_chars = "".to_string();
             let char_face = if caps["face"].is_empty() {attr_typeface} else {&caps["face"]};
-            let typeface_changed =  &typeface != char_face;
+            let typeface_changed =  typeface != char_face;
             if typeface_changed {
                 typeface = char_face.to_string();   // needs to outlast this instance of the loop
                 nemeth_chars += &typeface;
@@ -1276,13 +1340,13 @@ impl BrailleChars {
         // debug!("  result: {}", &result);
         let mut text_chars = text.chars();     // see if more than one char
         if is_all_caps_valid && is_all_caps && text_chars.next().is_some() &&  text_chars.next().is_some() {
-            return Ok( "CC".to_string() + &result.replace("C", ""));
+            return Ok( "CC".to_string() + &result.replace('C', ""));
         } else {
             return Ok( result.to_string() );
         }
     }
 
-    fn get_braille_ueb_chars<'a>(node: &'a Element, text_range: Option<Range<usize>>) -> StdResult<String, XPathError> {
+    fn get_braille_ueb_chars(node: &Element, text_range: Option<Range<usize>>) -> StdResult<String, XPathError> {
         // Because in UEB typeforms and caps may extend for multiple tokens,
         //   this routine merely deals with the mathvariant attr.
         // Canonicalize has already transformed all chars it can to math alphanumerics, but not all have bold/italic 
@@ -1290,12 +1354,12 @@ impl BrailleChars {
         lazy_static! {
             static ref HAS_TYPEFACE: Regex = Regex::new(".*?(double-struck|script|fraktur|sans-serif).*").unwrap();
             static ref PICK_APART_CHAR: Regex = 
-                 Regex::new(r"(?P<bold>B??)(?P<italic>I??)(?P<face>[S𝔹TD]??)s??(?P<cap>C??)(?P<char>[NL].)").unwrap();
+                 Regex::new(r"(?P<bold>B??)(?P<italic>I??)(?P<face>[S𝔹TD]??)s??(?P<cap>C??)(?P<greek>G??)(?P<char>[NL].)").unwrap();
         }
     
         let math_variant = node.attribute_value("mathvariant");
         let text = BrailleChars::substring(as_text(*node), text_range);
-        let braille_chars = crate::speech::braille_replace_chars(&text, *node).unwrap_or("".to_string());
+        let braille_chars = crate::speech::braille_replace_chars(&text, *node).unwrap_or_else(|_| "".to_string());
 
         if math_variant.is_none() {         // nothing we need to do
             return Ok(braille_chars);
@@ -1316,15 +1380,15 @@ impl BrailleChars {
             },
         };
         let result = PICK_APART_CHAR.replace_all(&braille_chars, |caps: &Captures| {
-            debug!("captures: {:?}", caps);
+            // debug!("captures: {:?}", caps);
             // debug!("  bold: {:?}, italic: {:?}, face: {:?}, cap: {:?}, char: {:?}",
             //        &caps["bold"], &caps["italic"], &caps["face"], &caps["cap"], &caps["char"]);
-            let new_char = if bold || !caps["bold"].is_empty() {"B"} else {""}.to_string()
+            if bold || !caps["bold"].is_empty() {"B"} else {""}.to_string()
                 + if italic || !caps["italic"].is_empty() {"I"} else {""}
                 + if !&caps["face"].is_empty() {&caps["face"]} else {typeface}
                 + &caps["cap"]
-                + &caps["char"];
-            new_char
+                + &caps["greek"]
+                + &caps["char"]
         });
         return Ok(result.to_string())
     }
@@ -1404,7 +1468,7 @@ impl Function for BrailleChars {
                             -> StdResult<Value<'d>, XPathError>
         {
             let mut args = Args(args);
-            if let Err(e) = args.exactly(2).or(args.exactly(4)) {
+            if let Err(e) = args.exactly(2).or_else(|_| args.exactly(4)) {
                 return Err( XPathError::Other(format!("BrailleChars requires 2 or 4 args: {}", e)));
             };
 
@@ -1427,3 +1491,33 @@ impl Function for BrailleChars {
         }
     }
     
+#[cfg(test)]
+mod tests {
+    use super::*;
+    #[allow(unused_imports)]
+    use crate::init_logger;
+    use crate::interface::*;
+    
+    #[test]
+    fn ueb_highlight_24() -> Result<()> {       // issue 24
+        init_logger();
+        let mathml_str = "<math display='block' id='M8o41h70-0'>
+                <mrow id='M8o41h70-1'>
+                <mn id='M8o41h70-2'>4</mn>
+                <mo id='M8o41h70-3'>&#x2062;</mo>
+                <mi id='M8o41h70-4'>a</mi>
+                <mo id='M8o41h70-5'>&#x2062;</mo>
+                <mi id='M8o41h70-6'>c</mi>
+            </mrow>
+        </math>";
+        crate::interface::set_rules_dir(super::super::abs_rules_dir_path()).unwrap();
+        set_mathml(mathml_str.to_string()).unwrap();
+        set_preference("BrailleCode".to_string(), "UEB".to_string()).unwrap();
+        set_preference("BrailleNavHighlight".to_string(), "All".to_string()).unwrap();
+        let braille = get_braille("M8o41h70-2".to_string())?;
+        assert_eq!("⣼⣙⠰⠁⠉", braille);
+        let braille = get_braille("M8o41h70-4".to_string())?;
+        assert_eq!("⠼⠙⣰⣁⠉", braille);
+        return Ok( () );
+    }
+}
\ No newline at end of file
